@@ -19,7 +19,7 @@ This module provides API web-server functionality
 =head1 OPTIONS
 
     sub startup {
-        my $self = shift->SUPER::startup( OPTION_NAME => VALUE, ... );
+        my $self = shift->SUPER::startup( NAME => VALUE, ... );
 
         # ...
     }
@@ -34,6 +34,17 @@ This option enables all of the init_* options, which are described bellow
 
 Default: off
 
+=head2 config_opts
+
+    config_opts => { ... }
+
+This option sets L<WWW::Suffit::Plugin::ConfigGeneral> plugin options
+
+Default:
+
+    `noload => 1` if $self->configobj exists
+    `defaults => $self->config` if $self->config is not void
+
 =head2 init_authdb
 
     init_authdb => 'on'
@@ -58,17 +69,6 @@ This option enables RSA keys initialize
 
 Default: off
 
-=head2 config_opts
-
-    config_opts => { ... }
-
-This option sets L<WWW::Suffit::Plugin::ConfigGeneral> plugin options
-
-Default:
-
-    `noload => 1` if $self->configobj exists
-    `defaults => $self->config` if $self->config is not void
-
 =head2 syslog_opts
 
     syslog_opts => { ... }
@@ -83,54 +83,106 @@ Default:
 
 This class implements the following attributes
 
+=head2 accepts
+
+    accepts => 0,
+
+Maximum number of connections a worker is allowed to accept, before stopping
+gracefully and then getting replaced with a newly started worker,
+passed along to L<Mojo::IOLoop/max_accepts>
+
+Default: 10000
+
+See L<Mojo::Server::Prefork/accepts>
+
 =head2 cache
 
 The Mojo::Cache object
 
+=head2 clients
+
+    clients => 0,
+
+Maximum number of accepted connections this server is allowed to handle concurrently,
+before stopping to accept new incoming connections, passed along to L<Mojo::IOLoop/max_connections>
+
+Default: 1000
+
+See L<Mojo::Server::Daemon/max_clients>
+
 =head2 configobj
 
 The Config::General object or undef
 
+=head2 acruxconfig
+
+The Acrux::Config object or undef
+
 =head2 datadir
 
+    datadir => '/var/lib/myapp',
+
 The sharedstate data directory (data dir)
 
 Default: /var/lib/<MONIKER>
 
 =head2 debugmode
 
+    debugmode => 0,
+
 If this attribute is enabled then this server is no daemonize performs
 
 =head2 documentroot
 
+    documentroot => '/var/www/myapp',
+
 Document root directory
 
 Default: /var/www/<MONIKER>
 
+=head2 gid
+
+    gid => 1000,
+    gid => getgrnam( 'anonymous' ),
+
+This attribute pass GID to set the real group identifier and the effective group identifier for this process
+
 =head2 homedir
 
+    homedir => '/usr/share/myapp',
+
 The Project home directory
 
 Default: /usr/share/<MONIKER>
 
 =head2 logfile
 
+    logfile => '/var/log/myapp.log',
+
 The log file
 
 Default: /var/log/<MONIKER>.log
 
 =head2 loglevel
 
+    loglevel => 'warn',
+
 This attribute performs set the log level
 
 Default: warn
 
 =head2 max_history_size
 
-Maximum number of logged messages to store in "history", defaults to 25
+    max_history_size => 25,
+
+Maximum number of logged messages to store in "history"
+
+Default: 25
 
 =head2 moniker
 
+    moniker => 'myapp',
+
 Project name in lowercase notation, project nickname, moniker.
 This value often used as default filename for configuration files and the like
 
@@ -140,32 +192,117 @@ See L<Mojolicious/moniker>
 
 =head2 mysecret
 
+    mysecret => 'dgdfg',
+
 Default secret string
 
 Default: <DEFAULT_SECRET>
 
+=head2 no_daemonize
+
+    no_daemonize => 1,
+
+This attribute disables the daemonize process
+
+Default: 0
+
+=head2 pidfile
+
+    pidfile => '/var/run/myapp.pid',
+
+The pid file
+
+Default: /tmp/prefork.pid
+
+See L<Mojo::Server::Prefork/pid_file>
+
 =head2 project_name
 
-The project name. For example: MyTestApp
+    project_name => 'MyApp',
+
+The project name. For example: MyApp
 
 Default: current class name
 
 =head2 private_key
 
+    private_key => '...',
+
 Private RSA key
 
 =head2 project_version
 
+    project_version => '0.01',
+
 The project version. For example: 1.00
 
 B<NOTE!> This is required attribute!
 
 =head2 public_key
 
+    public_key => '...',
+
 Public RSA key
 
+=head2 requests
+
+    requests => 0,
+
+Maximum number of keep-alive requests per connection
+
+Default: 100
+
+See L<Mojo::Server::Daemon/max_requests>
+
+=head2 reload_sig
+
+    reload_sig => 'USR2',
+    reload_sig => 'HUP',
+
+The signal name that will be used to receive reload commands from the system
+
+Default: USR2
+
+=head2 server_addr
+
+    server_addr => '*',
+
+Main listener address (host)
+
+Default: * (::0, 0:0:0:0)
+
+=head2 server_port
+
+    server_port => 8080,
+
+Main listener port
+
+Default: 8080
+
+=head2 server_url
+
+    server_url => 'http://127.0.0.1:8080',
+
+Main real listener URL
+
+See C<ListenAddr> and C<ListenPort> configuration directives
+
+Default: http://127.0.0.1:8080
+
+=head2 spare
+
+    spare => 0,
+
+Temporarily spawn up to this number of additional workers if there is a need.
+
+Default: 2
+
+See L<Mojo::Server::Prefork/spare>
+
 =head2 tempdir
 
+    tempdir => '/tmp/myapp',
+
 The temp directory
 
 Default: /tmp/<MONIKER>
@@ -176,25 +313,22 @@ List of trusted proxies
 
 Default: none
 
-=head2 server_addr
+=head2 uid
 
-Main listener address (host)
+    uid => 1000,
+    uid => getpwnam( 'anonymous' ),
 
-Default: * (::0, 0:0:0:0)
+This attribute pass UID to set the real user identifier and the effective user identifier for this process
 
-=head2 server_port
+=head2 workers
 
-Main listener port
+    workers => 0,
 
-Default: 8080
+Number of worker processes
 
-=head2 server_url
+Default: 4
 
-Main listener URL
-
-See ListenAddr and ListenPort configuration directives
-
-Default: http://127.0.0.1:8080
+See L<Mojo::Server::Prefork/workers>
 
 =head1 METHODS
 
@@ -204,12 +338,22 @@ This class inherits all methods from L<Mojolicious> and implements the following
 
 This method returns server listeners as list of URLs
 
-    $prefork->listen($app->listeners);
+    $prefork->listen( $app->listeners );
+
+=head2 preforked_run
+
+    $app->preforked_run( 'start' );
+    $app->preforked_run( 'stop' );
+    $app->preforked_run( 'restart' );
+    $app->preforked_run( 'status' );
+    $app->preforked_run( 'reload' );
+
+This method runs your application using a command that is passed as a argument
 
 =head2 raise
 
-    $self->raise("Mask %s", "val");
-    $self->raise("val");
+    $app->raise("Mask %s", "val");
+    $app->raise("val");
 
 Prints error message to STDERR and exit with errorlevel = 1
 
@@ -268,22 +412,25 @@ See C<LICENSE> file and L<https://dev.perl.org/licenses/>
 
 =cut
 
-our $VERSION = '1.07';
+our $VERSION = '1.08';
 
 use Mojo::Base 'Mojolicious';
 
 use Carp qw/carp croak/;
+use POSIX qw//;
 use File::Spec;
-use Mojo::URL;
 
+use Mojo::URL;
 use Mojo::Cache;
 use Mojo::File qw/ path /;
 use Mojo::Home qw//;
-use Mojo::Util qw/decamelize/; # decamelize(ref($self))
+use Mojo::Util qw/decamelize steady_time/; # decamelize(ref($self))
 use Mojo::Loader qw/load_class/;
 
+use Mojo::Server::Prefork;
+
 use WWW::Suffit::Const qw/
-        :general :security :session :dir
+        :general :security :session :dir :server
         AUTHDBFILE JWT_REGEXP
     /;
 use WWW::Suffit::Util qw/ color parse_time_offset /;
@@ -298,25 +445,45 @@ use constant {
         DEFAULT_SERVER_PORT => 8080,
     };
 
+# Common attributes
 has 'project_name';         # Anonymous
 has 'project_version';      # 1.00
 has 'server_url';           # http://127.0.0.1:8080
 has 'server_addr';          # * (0.0.0.0)
 has 'server_port';          # 8080
 has 'debugmode';            # 0
-has 'loglevel' => 'warn';   # warn
 has 'configobj';            # Config::General object
-has 'documentroot';         # /var/www/suffit-api
-has 'homedir';              # /usr/share/suffit-api
-has 'datadir';              # /var/lib/suffit-api
-has 'tempdir';              # /tmp/suffit-api
-has 'logfile';              # /var/log/suffit-api.log
+has 'acruxconfig';          # Acrux::Config object
+has 'cache' => sub { Mojo::Cache->new };
+
+# Files and directories
+has 'documentroot';         # /var/www/<MONIKER>
+has 'homedir';              # /usr/share/<MONIKER>
+has 'datadir';              # /var/lib/<MONIKER>
+has 'tempdir';              # /tmp/<MONIKER>
+has 'logfile';              # /var/log/<MONIKER>.log
+has 'pidfile';              # /run/<MONIKER>.pid
+
+# Logging
+has 'loglevel' => 'warn';   # warn
+has 'max_history_size' => MAX_HISTORY_SIZE;
+
+# Security
 has 'mysecret' => DEFAULT_SECRET; # Secret
 has 'private_key' => '';    # Private RSA key
 has 'public_key' => '';     # Public RSA key
 has 'trustedproxies' => sub { [grep {length} @{(shift->conf->list("/trustedproxy"))}] };
-has 'max_history_size' => MAX_HISTORY_SIZE;
-has 'cache' => sub { Mojo::Cache->new };
+
+# Prefork
+has 'clients' => sub { shift->conf->latest("/clients") || SERVER_MAX_CLIENTS }; # 10000
+has 'requests' => sub { shift->conf->latest("/requests") || SERVER_MAX_REQUESTS}; # 100
+has 'accepts' => sub { shift->conf->latest("/accepts") }; # SERVER_ACCEPTS
+has 'spare' => sub { shift->conf->latest("/spare") || SERVER_SPARE }; # 2
+has 'workers' => sub { shift->conf->latest("/workers") || SERVER_WORKERS }; # 4
+has 'reload_sig' => sub { shift->conf->latest("/reload_sig") // 'USR2' };
+has 'no_daemonize';
+has 'uid';
+has 'gid';
 
 sub raise {
     my $self = shift;
@@ -334,12 +501,25 @@ sub startup {
     push @{$self->routes->namespaces}, 'WWW::Suffit::Server'; # Add Server routes namespace
     my $all_features = is_true_flag($opts->{all_features}); # on/off
 
-    # ConfigGeneral
+    # Get all ConfigGeneral configuration attributes
     my $config_opts = as_hash_ref($opts->{config_opts}) || {};
-    $self->raise("The `configobj` must be Config::General object")
-        unless !$self->configobj or ref($self->configobj) eq 'Config::General';
-    $self->config($self->configobj->getall) if $self->configobj; # Set config hash if configobj exist
-    unless (exists($config_opts->{noload})) { $config_opts->{noload} = $self->configobj ? 1 : 0 }
+    if (my $configobj = $self->configobj) {
+        $self->raise("The `configobj` must be Config::General object")
+            unless ref($configobj) eq 'Config::General';
+        $self->config($configobj->getall); # Set config hash
+        $config_opts->{noload} = 1 unless exists $config_opts->{noload};
+    }
+
+    # Get all Acrux configuration attributes
+    if (my $acruxconfig = $self->acruxconfig) {
+        $self->raise("The `acruxconfig` must be Acrux::Config object")
+            unless ref($acruxconfig) eq 'Acrux::Config';
+        $self->config($acruxconfig->config); # Set config hash
+        $config_opts->{noload} = 1 unless exists $config_opts->{noload};
+    }
+
+    # Init ConfigGeneral plugin
+    unless (exists($config_opts->{noload})) { $config_opts->{noload} = 0 }
     unless (exists($config_opts->{defaults})) { $config_opts->{defaults} = as_hash_ref($self->config) }
     $self->plugin('ConfigGeneral' => $config_opts);
 
@@ -503,6 +683,132 @@ sub listeners {
     # Return listeners
     return [@listeners];
 }
+sub preforked_run {
+    my $self = shift; # app
+    my $dash_k = shift || '';
+
+    # Dash k
+    $self->raise("Incorrect LSB command! Please use start, status, stop, restart or reload")
+        unless (grep {$_ eq $dash_k} qw/start status stop restart reload/);
+
+    # Mojolicious Prefork server
+    my $prefork = Mojo::Server::Prefork->new( app => $self );
+       $prefork->pid_file($self->pidfile) if length $self->pidfile;
+
+    # Hypnotoad Pre-fork settings
+    $prefork->max_clients($self->clients) if defined $self->clients;
+    $prefork->max_requests($self->requests) if defined $self->requests;
+    $prefork->accepts($self->accepts) if defined $self->accepts;
+    $prefork->spare($self->spare) if defined $self->spare;
+    $prefork->workers($self->workers) if defined $self->workers;
+
+    # Listener
+    $prefork->listen($self->listeners);
+
+    # Working with Dash k
+    my $upgrade = 0;
+    my $reload = 0;
+    my $upgrade_timeout = SERVER_UPGRADE_TIMEOUT; # 30
+    if ($dash_k eq 'start') {
+        if (my $pid = $prefork->check_pid()) {
+            print "Already running $pid\n";
+            exit 0;
+        }
+    } elsif ($dash_k eq 'stop') {
+        if (my $pid = $prefork->check_pid()) {
+            kill 'QUIT', $pid;
+            print "Stopping $pid\n";
+        } else {
+            print "Not running\n";
+        }
+        exit 0;
+    } elsif ($dash_k eq 'restart') {
+        if (my $pid = $prefork->check_pid()) {
+            $upgrade ||= steady_time;
+            kill 'QUIT', $pid;
+            my $up = $upgrade_timeout;
+            while (kill 0, $pid) {
+                $up--;
+                sleep 1;
+            }
+            die("Can't stop $pid") if $up <= 0;
+            print "Stopping $pid\n";
+            $upgrade = 0;
+        }
+    } elsif ($dash_k eq 'reload') {
+        my $pid = $prefork->check_pid();
+        if ($pid) {
+            if (my $s = $self->reload_sig) {
+                # Start hot deployment
+                kill $s, $pid; # 'USR2'
+                print "Reloading $pid\n";
+                exit 0;
+            }
+        }
+        print "Not running\n";
+    } else { # status
+        if (my $pid = $prefork->check_pid()) {
+            print "Running $pid\n";
+        } else {
+            print "Not running\n";
+        }
+        exit 0;
+    }
+
+    # Listen USR2 (reload)
+    if (my $s = $self->reload_sig) {
+        $SIG{$s} = sub { $upgrade ||= steady_time };
+    }
+
+    # Set system hooks
+    $prefork->on(wait => sub { # Manage (every 1 sec)
+        my $this = shift; # Prefork object
+
+        # Upgrade
+        if ($upgrade) {
+            #$this->app->log->debug(">>> " . $this->healthy() || '?');
+            unless ($reload) {
+                $reload = 1; # Off next reloading
+                if ($this->app->reload()) {
+                    $reload = 0;
+                    $upgrade = 0;
+                    return 1;
+                }
+            }
+
+            # Timeout
+            if (($upgrade + $upgrade_timeout) <= steady_time()) {
+                kill 'KILL', $$;
+                $upgrade = 0;
+            }
+        }
+    });
+    $prefork->on(finish => sub { # Finish
+        my $this = shift; # Prefork object
+        my $graceful = shift;
+        $this->app->log->debug($graceful ? 'Graceful server shutdown' : 'Server shutdown');
+    });
+
+    # Set GID and UID
+    if (IS_ROOT) {
+        if (my $gid = $self->gid) {
+            POSIX::setgid($gid) or $self->raise("setgid %s failed - %s", $gid, $!);
+            $) = "$gid $gid"; # this calls setgroups
+            $self->raise("detected strange gid") if !($( eq "$gid $gid" && $) eq "$gid $gid"); # just to be sure
+        }
+        if (my $uid = $self->uid) {
+            POSIX::setuid($uid) or $self->raise("setuid %s failed - %s", $uid, $!);
+            $self->raise("detected strange uid") if !($< == $uid && $> == $uid); # just to be sure
+        }
+    }
+
+    # Daemonize
+    $prefork->daemonize() unless $self->no_daemonize;
+
+    # Running
+    print "Running\n";
+    $prefork->run();
+}
 
 sub _load_module {
     my $module = shift;
